diff -rupN strace-4.7/signal.c strace-4.7-patched/signal.c
--- strace-4.7/signal.c	2012-04-16 20:38:17.000000000 -0600
+++ strace-4.7-patched/signal.c	2013-02-08 17:45:35.364791504 -0700
@@ -1142,8 +1142,9 @@ static int
 print_stack_t(struct tcb *tcp, unsigned long addr)
 {
 	stack_t ss;
-	if (umove(tcp, addr, &ss) < 0)
-		return -1;
+	int ret;
+	if ((ret = umove(tcp, addr, &ss)) < 0)
+		return ret;
 	tprintf("{ss_sp=%#lx, ss_flags=", (unsigned long) ss.ss_sp);
 	printflags(sigaltstack_flags, ss.ss_flags, "SS_???");
 	tprintf(", ss_size=%lu}", (unsigned long) ss.ss_size);
@@ -1154,17 +1155,22 @@ int
 sys_sigaltstack(struct tcb *tcp)
 {
 	if (entering(tcp)) {
-		if (tcp->u_arg[0] == 0)
+		int ret;
+		if (tcp->u_arg[0] == 0) {
 			tprints("NULL");
-		else if (print_stack_t(tcp, tcp->u_arg[0]) < 0)
+		}
+		else if ((ret = print_stack_t(tcp, tcp->u_arg[0])) == -1)
 			return -1;
+		/* if ret == -2 (invalid address), pass through. */
 	}
 	else {
+		int ret;
 		tprints(", ");
 		if (tcp->u_arg[1] == 0)
 			tprints("NULL");
-		else if (print_stack_t(tcp, tcp->u_arg[1]) < 0)
+		else if ((ret = print_stack_t(tcp, tcp->u_arg[1])) == -1)
 			return -1;
+		/* if ret == -2 (invalid address), pass through. */
 	}
 	return 0;
 }
diff -rupN strace-4.7/util.c strace-4.7-patched/util.c
--- strace-4.7/util.c	2012-04-28 06:58:35.000000000 -0600
+++ strace-4.7-patched/util.c	2013-02-08 17:42:34.978118913 -0700
@@ -805,9 +805,14 @@ umoven(struct tcb *tcp, long addr, int l
 		u.val = ptrace(PTRACE_PEEKDATA, pid, (char *) addr, 0);
 		if (errno) {
 			/* But if not started, we had a bogus address. */
-			if (addr != 0 && errno != EIO && errno != ESRCH)
+			int ret = -1;
+			if (addr != 0 && (errno == EIO || errno == EFAULT)) {
+				ret = -2;
+			}
+			if (addr != 0 && errno != EIO && errno != ESRCH) {
 				perror_msg("umoven: PTRACE_PEEKDATA pid:%d @0x%lx", pid, addr);
-			return -1;
+			}
+			return ret;
 		}
 		started = 1;
 		m = MIN(sizeof(long) - n, len);
